/*
 * MX 68HC11 Multitasking Executive Example Program 4
 * Demonstrates buffered serial port access
 */
#include \mc\stdio.h
#include \mc\lib11\mx.h
#include ex.h

int error;
char *task1h, *task2h, *task3h, *mainh, *sema, c, i;

task1(message)
int message;
{
int i;
switch(message){
   case 1: /* Output 32 characters */
            for (i=0;i<32;i++) Cputc('1');
            break;
   case 2: /* Output three strings */
            Cputstr("Task 1 printing string 1\n");
            Cputstr("Task 1 printing string 2\n");
            Cputstr("Task 1 printing string 3\n");
            break;
   default: Cprintf("Message received value %d\n", message); 
   }
}

task2(message)
int message;
{
int i;
switch(message){
   case 1: /* Output 32 characters */
            for (i=0;i<32;i++) Cputc('2');
            break;
   case 2: /* Output three strings */
            Cputstr("Task 2 printing string 1\n");
            Cputstr("Task 2 printing string 2\n");
            Cputstr("Task 2 printing string 3\n");
            break;
   default: Cprintf("Message received value %d\n", message); 
   }
}

task3(message)
int message;
{
switch (message){
   case 1:  sleep(10);
            sendmsg(task1h, 1);
            sendmsg(task2h, 1);
            sleep(10);
            break;

   case 2:  sleep(10);
            sendmsg(task1h, 2);
            sendmsg(task2h, 2);
            break;
   }
}

demoCputc() 
{
int i,c;
Cputstr("\nUsing CputC() to output characters\n"); 
c=32;
for(i=0;i<64;i++) Cputc('A'+i);
}

demoCputstr()
{
Cputstr("\nUsing Cputstr() to output strings\n");
Cputstr("This is a test string to output by Cputstr()\n");
}

demoCprintf()
{
Cputstr("\nUsing Cprintf()\n");
Cprintf("This is using Cprintf()\n");
Cprintf("This is using Cprintf() to output numbers %d %x\n", 100, 127);
}

demoCtestc()
{
int c;
Cputstr("\nUsing Ctestc()\n");
Cprintf("Value Ctestc() returned with no character available %d\n", Ctestc());
Cputstr("Please press any key to demonstrate Ctestc(), press <ESC> to exit\n");

do {
   c = Ctestc();
   if (c != -1) Cprintf("Value returned from Ctestc() is %d\n", c);
   }  while (c !=27);
}

demoCgetcr()
{
int c;
Cputstr("\nUsing Cgetcr()\n");
Cputstr("Please press any key to demonstrate Cgetcr(), press <ESC> to exit\n");

do {
   c = Cgetcr(20);
   Cprintf("Value returned from Ctestc(20) is %d\n", c);
   }  while (c !=27);
}

demoCputcr() 
{
int i,c;
Cputstr("\nUsing Cputcr() to output characters\n"); 
c=32;
for(i=0;i<64;i++) Cputcr('A'+i);
}

demomult1()
{
Cputstr("Demonstrating multiple tasks using Cputc()\n");
Cputstr("Note. Interleaving of printed characters after buffer fills\n\n");
sendmsg(task3h,1); /* Use task 3 as proxy */
sleep(20);
Cputstr("\n");
}

demomult2()
{
Cputstr("Demonstrating multiple tasks using Cputstr()/Cprintf()\n\n");
Cputstr("Note: Only complete strings are printed\n\n");
sendmsg(task3h,2); /* Use task 3 as a proxy */
}

showmenu()
{
Cputstr("\n\n  MX-11 Communication Driver Example program\n\n");
Cputstr("  1            Cputc()\n");
Cputstr("  2            Cputstr()\n");
Cputstr("  3            Cprintf()\n");
Cputstr("  4            Ctestc()\n");
Cputstr("  5            Multiple tasks using Cputc\n");
Cputstr("  6            Multiple tasks using Cprintf()\n");
Cputstr("  7            Cputcr()\n");
Cputstr("  8            Cgetcr()\n");
Cputstr("  T            Show tasks\n\n");
Cputstr("Press selection,  press <ESC> to exit...");
}

showtasks()
{
/* Follow TCB's and display Tasks */
Cprintf("MX Variables start at %4x\n", mxvar());
pTCB=mxvar();
Cprintf(
"Task                TCB   Start Stack Status    Pri SPend SPmin Space\n");
Cprintf(
"----                ---   ----- ----- ------    --- ----- ----- -----\n");

while (pTCB=pTCB->tlink)
   {
   Cprintf("%-18s  %4x  %4x  %4x  %s  %d  %4x  %4x   %d\n",
   pTCB->tname, pTCB, pTCB->tstart, pTCB->tsp, Status[pTCB->tstat],
   pTCB->tpriority,pTCB->tspend,pTCB->tspmin,(pTCB->tspmin-pTCB->tspend));
   }
Cputc('\n');
}

main()
{
/* Initialize MX for 7 tasks, 32 MCB's and 256 byte stack space for main */
mainh=mxinit(256,7,32);
mxstkchk();
Copen(20,20);

task1h=deftask( &task1,256,"Task 1"); /* define task1 */ 
task2h=deftask( &task2,256,"Task 2"); /* define task2 */ 
task3h=deftask( &task3,256,"Task 3"); /* define task3 */ 

/* Main now waits for Commands until escape pressed */
do {
   showmenu();
   c=Cgetc(); Cputc('\n');
   switch (c) {
      case '1':demoCputc(); break;
      case '2':demoCputstr(); break;
      case '3':demoCprintf(); break;
      case '4':demoCtestc(); break;
      case '5':demomult1(); break;
      case '6':demomult2(); break;
      case '7':demoCputcr(); break;
      case '8':demoCgetcr(); break;

      case 'T':
      case 't':showtasks(); break;
      }
sleep(20);
if (c!=27) {Cputstr("\nmain(): Press any key to continue..."); Cgetc();Cputc('\n');
   }
}
while (c !=27); /* Repeat until escape key pressed */
Cputstr("\nProgram finished\n");
sleep(20);
disable();
}
